==> firebase-template-0.2.0.tgz: package/index.js <==
var Delegate = require('dom-delegate');
var Firebase = require('firebase');
var insertCss = require('insert-css');
var prop = require('propper');
var path = require('path');
var store = require('store');

var resize = require('./lib/resize.js');

module.exports = FirebaseTemplate;

function FirebaseTemplate(options) {
  options = options || {};

  var root = options.el || document.querySelector('body');

  var rootPath = root.dataset.path;
  delete root.dataset.path;
  if (!rootPath)
    throw new Error('data-path not set not set on target');

  var oldData;
  var newData;

  var firebaseRef = new Firebase(rootPath);

  // Make images displaying firebase content appear as clickable
  insertCss('img[data-path] { cursor: pointer; }');

  var authToken = root.dataset.jwt;
  if (authToken) {
    firebaseRef.authWithCustomToken(authToken, function (err, authData) {
      if (err) {
        throw new Error('Auth Failed ' + err);
      }

      // console.log("Login Succeeded!", authData);
    });
  } else {
    console.warn('data-jwt not given, so assuming anonymous access');
  }

  var dataRef = firebaseRef.child('data');
  var uploadsRef = firebaseRef.child('uploads');
  var delegate = new Delegate(root);

  var lazyUpdateDomCollections = lazyUpdate(updateDomCollections, function (obj) {
    return Object.keys(obj || {}).join();
  });

  var lazyUpdateDomImgs = lazyUpdate(updateDomImgs, function (val) {
    return val ? val.lastModified : null;
  });

  var lazyUpdateDomInputs = lazyUpdate(updateDomInputs);
  var lazyUpdateDomSelects = lazyUpdate(updateDomSelects);

  return {
    start: start,
    stop: stop
  };

  function stop() {
    delegate.off();
    dataRef.off('value');

    oldData = null;
    newData = {};
  }

  function start() {
    oldData = null;
    newData = store.get('data:' + rootPath) || {};

    updateDom();

    dataRef.on('value', function (snapshot) {
      newData = snapshot.val();
      store.set('data:' + rootPath, newData);
      updateDom();
    });

    registerDomListeners();
  }

  function lazyUpdate(updater, hash) {
    hash = hash || identity;

    return function (el, newValue) {
      var oldValue = prop(oldData, extractFbPath(el));
      if (hash(newValue) !== hash(oldValue)) {
        updater(el, newValue);
      }
    };
  }

  function updateDom() {
    // expand arrays to fit the content
    applyDomChanges('[data-type=collection][data-path]', lazyUpdateDomCollections);

    applyDomChanges('img[data-path]', lazyUpdateDomImgs);
    applyDomChanges('input[data-path],textarea[data-path]', lazyUpdateDomInputs);
    applyDomChanges('[data-show]', function (el, newValue) {
      el.style.display = newValue ? '' : 'none';
    });
    applyDomChanges('select[data-path]', lazyUpdateDomSelects);

    oldData = newData;
  }

  function updateDomCollections(el, newValue) {
    var path = extractFbPath(el);

    //force the fields to reload their content in the collection
    if (oldData !== null) {
      prop(oldData, path, {});
    }

    var childTemplate = el.dataset.childTemplate;
    if (!childTemplate) {
      childTemplate = el.dataset.childTemplate = el.innerHTML;
    }

    el.innerHTML = Object.keys(newValue || {}).map(function (id) {
      return childTemplate.replace(/data-(path|show)="([^"]*)"/g, function (match, p, path) {
        return 'data-' + p + '="' + id + (path === '.' ? '' : '/' + path) + '"';
      });
    }).join('');
  }

  function updateDomImgs(el, newValue) {
    var fbPath = extractFbPath(el);

    el.style.display = newValue ? '' : 'none';
    if (newValue) {
      var thumbKey = 'thumb:' + newValue.lastModified + ':' + fbPath;
      var thumbSrc = store.get(thumbKey);
      if (thumbSrc) {
        el.src = thumbSrc;
      } else {
        uploadsRef.child(fbPath + '/thumb').once('value', function (snapshot) {
          el.src = store.set(thumbKey, snapshot.val());
        });
      }
    }
  }

  function updateDomInputs(el, newValue) {
    switch (el.type) {
      case 'file':
        break;
      case 'checkbox':
        el.checked = newValue === 1;
        break;
      default:
        el.value = newValue || '';
    }
  }

  function updateDomSelects(el, newValue) {
    var options = el.options;
    // reset selections
    for (var i = 0; i < options.length; i++) {
      var option = options[i];
      option.selected = (option.value == newValue);
    }
  }

  function registerDomListeners() {
    delegate.off();

    delegate.on('click', 'img[data-path]', function (e) {
      uploadsRef.child(extractFbPath(e.target) + '/raw').once('value', function (snapshot) {
        var src = snapshot.val();
        window.open(src, '_blank');
      });
    });

    delegate.on('click', '[data-action=add][data-path]', function (e) {
      dataRef.child(extractFbPath(e.target)).push({added: Date.now()});
    });

    delegate.on('click', '[data-action=remove][data-path]', function (e) {
      if (confirm('Are you sure?')) {
        var fbPath = extractFbPath(e.target);

        dataRef.child(fbPath).remove(function () {
          uploadsRef.child(fbPath).remove();
        });
      }
    });

    delegate.on('change', 'input[type=file][data-path]', function (e) {
      var fbPath = extractFbPath(e.target);
      var file = e.target.files[0];
      var reader = new FileReader();

      reader.onload = function (e) {
        var fileMeta = {
          lastModified: file.lastModified,
          name: file.name,
          size: file.size,
          type: file.type
        };
        if (file.type.match(/^image\//)) {
          var thumb = resize(e.target.result, 100);
          uploadsRef.child(fbPath + '/thumb').set(thumb, function () {
            dataRef.child(fbPath).set(fileMeta, function () {
              uploadsRef.child(fbPath + '/raw').set(e.target.result);
            });
          });
        } else {
          uploadsRef.child(fbPath).set(e.target.result, function () {
            dataRef.child(fbPath).set(fileMeta);
          });
        }
      };

      reader.readAsDataURL(file);
    });

    delegate.on('change', 'input[type=number][data-path]', function (e) {
      setData(e.target, Number(e.target.value));
    });

    delegate.on('change', 'input[type=checkbox][data-path]', function (e) {
      setData(e.target, e.target.checked ? 1 : 0);
    });

    delegate.on('change', 'select[data-path]', function(e) {
      setData(e.target, e.target.options[e.target.selectedIndex].value);
    });

    delegate.on('change', [
      'input:not([type])[data-path]',
      'input[type=text][data-path]',
      'input[type=email][data-path]',
      'textarea[data-path]'
    ].join(','), function (e) {
      setData(e.target, e.target.value);
    });
  }

  function setData(el, value) {
    var valueRef = dataRef.child(extractFbPath(el));
    if (value === '') {
      valueRef.remove();
    } else {
      valueRef.set(value);
    }
  }

  function queryAll(selector) {
    return [].slice.call(root.querySelectorAll(selector));
  }

  function applyDomChanges(selector, applier) {
    queryAll(selector).forEach(function (el) {
      var path;
      if (el.dataset.show) {
        path = extractFbShowPath(el);
      } else {
        path = extractFbPath(el);
      }

      applier(el, prop(newData, path));
    });
  }
}

function extractFbPath(el) {
  var path = prop(el, 'dataset.path');

  if (!path || path.charAt(0) !== '/') {
    path = el.dataset.path = collapsePathParts(extractPathParts(el));
  }

  return path;
}

function extractFbShowPath(el) {
  var showPath = el.dataset.show;

  if (showPath.charAt(0) !== '/') {
    showPath = el.dataset.show = collapsePathParts(extractPathParts(el.parentNode).concat(showPath));
  }

  return showPath;
}

function extractPathParts(el) {
  var parts = [];
  do {
    parts.push(el.dataset.path);
    el = el.parentNode;
  } while (el && el.dataset);

  return parts.filter(Boolean).reverse();
}

function collapsePathParts(parts) {
  var cleanParts = [];

  parts.join('/').split('/').forEach(function (part) {
    if (part === '') {
      cleanParts = [];
    } else if (part === '..') {
      cleanParts.pop();
    } else if (part !== '.') {
      cleanParts.push(part);
    }
  });

  return '/' + cleanParts.join('/');
}

function identity(val) {
  return val;
}

==> firebase-template-0.5.2.tgz: package/index.js <==
var Delegate = require('dom-delegate');
var Firebase = require('firebase');
var insertCss = require('insert-css');
var prop = require('propper');
var path = require('path');
var store = require('store');

var resize = require('./lib/resize.js');

function FirebaseTemplate(options) {
  options = options || {};

  var transform = options.transform || identity;

  var root = options.el || document.querySelector('body');
  var rootPath = root.dataset.path;

  if (!rootPath) {
    throw new Error('data-path not set not set on target');
  }

  delete root.dataset.path;

  var oldData;
  var newData;

  var firebaseRef = new Firebase(rootPath);

  // Make images displaying firebase content appear as clickable
  insertCss('img[data-path] { cursor: pointer; }');

  var authToken = root.dataset.jwt;
  if (authToken) {
    firebaseRef.authWithCustomToken(authToken, function (err, authData) {
      if (err) {
        throw new Error('Auth Failed ' + err);
      }

      // console.log("Login Succeeded!", authData);
    });
  } else {
    console.warn('WARNING: data-jwt not given, so assuming anonymous access');
  }

  var dataRef = firebaseRef.child('data');
  var uploadsRef = firebaseRef.child('uploads');
  var delegate = new Delegate(root);

  var lazyUpdateDomCollections = lazyUpdate(updateDomCollections, function (obj) {
    return Object.keys(obj || {}).join();
  });

  var lazyUpdateDomImgs = lazyUpdate(updateDomImgs, function (val) {
    return val ? val.lastModified : null;
  });

  var lazyUpdateDomInputs = lazyUpdate(updateDomInputs);
  var lazyUpdateDomSpans = lazyUpdate(updateDomSpans);
  var lazyUpdateDomHrefs = lazyUpdate(updateDomHrefs);

  this.start = function start() {
    oldData = null;
    newData = store.get('data:' + rootPath) || {};

    updateDom(newData);

    dataRef.on('value', function (snapshot) {
      newData = snapshot.val();
      store.set('data:' + rootPath, newData);
      updateDom(newData);
    });

    registerDomListeners();
  };

  this.stop = function stop() {
    delegate.off();
    dataRef.off('value');

    oldData = null;
    newData = {};
  };


  function lazyUpdate(updater, hash) {
    hash = hash || identity;

    return function (el, newValue) {
      var oldValue = prop(oldData, extractFbPath(el));

      if (newValue === undefined || oldValue === undefined || hash(newValue) !== hash(oldValue)) {
        updater(el, newValue);
      }
    };
  }

  // exposed for unit testing
  this._updateDom = function(after, before) {
    newData = after;
    oldData = before || null;
    updateDom();
  };

  function updateDom() {
    newData = transform(newData) || newData;

    // expand arrays to fit the content
    applyDomChanges('[data-type=collection][data-path]', lazyUpdateDomCollections);
    applyDomChanges('img[data-path]', lazyUpdateDomImgs);
    applyDomChanges('input[data-path],textarea[data-path]', lazyUpdateDomInputs);
    applyDomChanges('span[data-path]', lazyUpdateDomSpans);
    applyDomChanges('[data-href]', lazyUpdateDomHrefs);
    applyDomChanges('[data-show]', function (el, newValue) {
      el.style.display = newValue ? '' : 'none';
    });

    oldData = newData;
  }

  function updateDomCollections(el, newValue) {
    newValue = newValue || {};

    var path = extractFbPath(el);

    //force the fields to reload their content in the collection
    if (oldData !== null) {
      prop(oldData, path, {});
    }

    var sortHash = function(item) {
      if (item.weight !== undefined) {
        return item.weight;
      }
      return item.added;
    };

    var sortedKeys = Object.keys(newValue).sort(function(a, b) {
      var aSortHash = sortHash(newValue[a]);
      var bSortHash = sortHash(newValue[b]);
      if (aSortHash === bSortHash) return 0;
      if (aSortHash < bSortHash) return -1;
      return 1;
    });

    var sortedValues = {};
    sortedKeys.forEach(function(key, index) {
      var value = newValue[key];
      value.index = index;
      sortedValues[key] = value;
    });

    var childTemplate = el.dataset.childTemplate;
    if (!childTemplate) {
      childTemplate = el.dataset.childTemplate = el.innerHTML;
    }

    el.innerHTML = Object.keys(sortedValues || {}).map(function (id) {
      return childTemplate.replace(/data-(path|show)="([^"]*)"/g, function (match, p, path) {
        return 'data-' + p + '="' + id + (path === '.' ? '' : '/' + path) + '"';
      });
    }).join('');
  }

  function updateDomImgs(el, newValue) {
    var fbPath = extractFbPath(el);

    el.style.display = newValue ? '' : 'none';
    if (newValue) {
      var thumbKey = 'thumb:' + newValue.lastModified + ':' + fbPath;
      var thumbSrc = store.get(thumbKey);
      if (thumbSrc) {
        el.src = thumbSrc;
      } else {
        uploadsRef.child(fbPath + '/thumb').once('value', function (snapshot) {
          el.src = store.set(thumbKey, snapshot.val());
        });
      }
    }
  }

  function updateDomInputs(el, newValue) {
    switch (el.type) {
      case 'file':
        break;
      case 'checkbox':
        el.checked = newValue === 1;
        break;
      default:
        el.value = newValue || '';
    }
  }

  function updateDomSpans(el, newValue) {
    el.innerText = newValue || '';
  }

  function updateDomHrefs(el, newValue) {
    el.href = newValue || '';
  }

  function registerDomListeners() {
    delegate.off();

    delegate.on('click', 'img[data-path]', function (e) {
      uploadsRef.child(extractFbPath(e.target) + '/raw').once('value', function (snapshot) {
        var src = snapshot.val();
        window.open(src, '_blank');
      });
    });

    delegate.on('click', '[data-action=add][data-path]', function (e) {
      dataRef.child(extractFbPath(e.target)).push({added: Date.now()});
    });

    delegate.on('click', '[data-action=remove][data-path]', function (e) {
      if (confirm('Are you sure?')) {
        var fbPath = extractFbPath(e.target);

        dataRef.child(fbPath).remove(function () {
          uploadsRef.child(fbPath).remove();
        });
      }
    });

    delegate.on('change', 'input[type=file][data-path]', function (e) {
      var fbPath = extractFbPath(e.target);
      var file = e.target.files[0];
      var reader = new FileReader();

      reader.onload = function (e) {
        var fileMeta = {
          lastModified: file.lastModified,
          name: file.name,
          size: file.size,
          type: file.type
        };
        if (file.type.match(/^image\//)) {
          var thumb = resize(e.target.result, 100);
          uploadsRef.child(fbPath + '/thumb').set(thumb, function () {
            dataRef.child(fbPath).set(fileMeta, function () {
              uploadsRef.child(fbPath + '/raw').set(e.target.result);
            });
          });
        } else {
          uploadsRef.child(fbPath).set(e.target.result, function () {
            dataRef.child(fbPath).set(fileMeta);
          });
        }
      };

      reader.readAsDataURL(file);
    });

    delegate.on('change', 'input[type=number][data-path]', function (e) {
      setData(e.target, Number(e.target.value));
    });

    delegate.on('change', 'input[type=checkbox][data-path]', function (e) {
      setData(e.target, e.target.checked ? 1 : 0);
    });

    delegate.on('change', [
      'input:not([type])[data-path]',
      'input[type=text][data-path]',
      'input[type=email][data-path]',
      'textarea[data-path]'
    ].join(','), function (e) {
      setData(e.target, e.target.value);
    });
  }

  function setData(el, value) {
    var valueRef = dataRef.child(extractFbPath(el));
    if (value === '') {
      valueRef.remove();
    } else {
      valueRef.set(value);
    }
  }

  function queryAll(selector) {
    return [].slice.call(root.querySelectorAll(selector));
  }

  function applyDomChanges(selector, applier) {
    queryAll(selector).forEach(function (el) {
      var path;
      if (el.dataset.show) {
        path = extractFbShowPath(el);
      } else if (el.dataset.href) {
        path = extractFbHrefPath(el);
      } else {
        path = extractFbPath(el);
      }

      applier(el, prop(newData, path));
    });
  }
}

function extractFbPath(el) {
  var path = prop(el, 'dataset.path') || prop(el, 'dataset.show') || prop(el, 'dataset.href');

  if (!path || path.charAt(0) !== '/') {
    path = el.dataset.path = collapsePathParts(extractPathParts(el));
  }

  return path;
}

function extractFbShowPath(el) {
  var showPath = el.dataset.show;

  if (showPath.charAt(0) !== '/') {
    showPath = el.dataset.show = collapsePathParts(extractPathParts(el.parentNode).concat(showPath));
  }

  return showPath;
}

function extractFbHrefPath(el) {
  var hrefPath = el.dataset.href;
  if (hrefPath.charAt(0) !== '/') {
    hrefPath = el.dataset.href = collapsePathParts(extractPathParts(el.parentNode).concat(hrefPath));
  }

  return hrefPath;
}

function extractPathParts(el) {
  var parts = [];
  do {
    parts.push(el.dataset.path);
    el = el.parentNode;
  } while (el && el.dataset);

  return parts.filter(Boolean).reverse();
}

function collapsePathParts(parts) {
  var cleanParts = [];

  parts.join('/').split('/').forEach(function (part) {
    if (part === '') {
      cleanParts = [];
    } else if (part === '..') {
      cleanParts.pop();
    } else if (part !== '.') {
      cleanParts.push(part);
    }
  });

  return '/' + cleanParts.join('/');
}

function identity(val) {
  return val;
}

module.exports = FirebaseTemplate;
